@@ -1,10 +1,8 @@
 -- KeyringParser.hs: OpenPGP (RFC4880) transferable keys parsing
--- Copyright © 2012-2016  Clint Adams
+-- Copyright © 2012-2018  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
-{-# LANGUAGE CPP #-}
-
 module Codec.Encryption.OpenPGP.KeyringParser (
  -- * Parsers
    parseAChunk
@@ -30,13 +28,7 @@ module Codec.Encryption.OpenPGP.KeyringParser (
 ) where
 
 import Control.Applicative (many, (<|>))
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>))
-#endif
 import Data.Maybe (catMaybes)
-#if !MIN_VERSION_base(4,8,0)
-import Data.Monoid (Monoid, mconcat)
-#endif
 import Data.Monoid ((<>))
 
 import Data.Text (Text)
