@@ -1,10 +1,8 @@
 -- Serialize.hs: OpenPGP (RFC4880) serialization (using cereal)
--- Copyright © 2012-2016  Clint Adams
+-- Copyright © 2012-2018  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
-{-# LANGUAGE CPP #-}
-
 module Codec.Encryption.OpenPGP.Serialize (
   -- * Serialization functions
    putSKAddendum
@@ -13,9 +11,6 @@ module Codec.Encryption.OpenPGP.Serialize (
  , parsePkts
 ) where
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>))
-#endif
 import Control.Applicative (many, some)
 import Control.Lens ((^.), _1)
 import Control.Monad (guard, replicateM, replicateM_)
@@ -36,9 +31,6 @@ import Data.Binary (Binary, get, put)
 import Data.Binary.Get (Get, getByteString, getLazyByteString, getRemainingLazyByteString, getWord8, getWord16be, getWord32be, getWord16le, runGetOrFail, ByteOffset)
 import Data.Binary.Put (Put, putWord8, putWord16be, putWord32be, putByteString, putLazyByteString, putWord16le, runPut)
 import qualified Data.Foldable as F
-#if !MIN_VERSION_base(4,8,0)
-import Data.Monoid (mempty)
-#endif
 import Data.Set (Set)
 import qualified Data.Set as Set
 import qualified Data.Text as T
@@ -1270,7 +1262,8 @@ putSignaturePayload (SigVOther pv bs) = do
 
 putTK :: TK -> Put
 putTK key = do
-    put (PublicKey (key^.tkKey._1))
+    let pkp = key^.tkKey._1
+    maybe (put (PublicKey pkp)) (\ska -> put (SecretKey pkp ska)) (snd (key^.tkKey))
     mapM_ (put . Signature) (_tkRevs key)
     mapM_ putUid' (_tkUIDs key)
     mapM_ putUat' (_tkUAts key)
