@@ -1,9 +1,8 @@
 -- Base.hs: OpenPGP (RFC4880) data types
--- Copyright © 2012-2016  Clint Adams
+-- Copyright © 2012-2018  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
-{-# LANGUAGE CPP #-}
 {-# LANGUAGE DataKinds #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
@@ -16,9 +15,6 @@ module Codec.Encryption.OpenPGP.Types.Internal.Base where
 
 import GHC.Generics (Generic)
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>))
-#endif
 import Control.Applicative ((<|>))
 import Control.Arrow ((***))
 import Control.Lens (makeLenses)
@@ -41,9 +37,6 @@ import Data.List.NonEmpty (NonEmpty)
 import qualified Data.List.NonEmpty as NE
 import Data.List.Split (chunksOf)
 import Data.Maybe (fromMaybe)
-#if !MIN_VERSION_base(4,8,0)
-import Data.Monoid (Monoid, mempty)
-#endif
 import Data.Monoid ((<>))
 import Data.Ord (comparing)
 import Data.Semigroup (Semigroup)
@@ -992,7 +985,7 @@ instance Pretty ImageFormat where
 
 $(ATH.deriveJSON ATH.defaultOptions ''ImageFormat)
 
-data ImageHeader = ImageHV1 ImageFormat
+newtype ImageHeader = ImageHV1 ImageFormat
     deriving (Data, Eq, Generic, Show, Typeable)
 
 instance Ord ImageHeader where
